@@ -24,15 +24,20 @@ limitations under the License.
 from __future__ import annotations
 
 import asyncio
+import logging
+import traceback
 from datetime import datetime
 from dataclasses import dataclass
 from typing import (
-    # TypeVar,
-    # cast,
+    # TYPE_CHECKING,
     Optional,
     Type,
-    # Union,
-    List, Union)
+    List,
+    Union
+)
+
+import requests.exceptions
+from grpclib.exceptions import StreamTerminatedError
 from pyee import AsyncIOEventEmitter  # type: ignore
 
 from wechaty_puppet import (  # type: ignore
@@ -53,14 +58,16 @@ from wechaty_puppet import (  # type: ignore
     ScanStatus,
     EventReadyPayload,
 
-    get_logger
+    WechatyPuppetError,
+
+    get_logger,
 )
 from wechaty_puppet.schemas.puppet import PUPPET_EVENT_DICT, PuppetOptions  # type: ignore
 from wechaty_puppet.state_switch import StateSwitch  # type: ignore
 from wechaty_puppet.watch_dog import WatchdogFood, Watchdog  # type: ignore
 
-from .plugin import (
-    WechatyPlugin, WechatyPluginManager
+from .utils import (
+    qr_terminal
 )
 
 from .user import (
@@ -76,11 +83,18 @@ from .user import (
     ContactSelf
 )
 
-from .utils import (
-    qr_terminal
+from .plugin import (
+    WechatyPlugin,
+    WechatyPluginManager
 )
 
-log = get_logger('Wechaty')
+from .exceptions import (  # type: ignore
+    WechatyStatusError,
+    WechatyConfigurationError,
+    WechatyOperationError,
+)
+
+log: logging.Logger = get_logger('Wechaty')
 
 DEFAULT_TIMEOUT = 300
 
@@ -121,6 +135,8 @@ class Wechaty(AsyncIOEventEmitter):
         if options.puppet_options is None:
             options.puppet_options = PuppetOptions()
 
+        self._options = options
+
         # pylint: disable=C0103
         self.Tag: Type[Tag] = Tag
         # pylint: disable=C0103
@@ -143,18 +159,29 @@ class Wechaty(AsyncIOEventEmitter):
 
         self.started: bool = False
 
-        self.puppet: Puppet = self._load_puppet(options)
-
         self._name: Optional[str] = None
 
         self.state = StateSwitch()
         self._ready_state = StateSwitch()
 
+        self._puppet: Optional[Puppet] = None
+
         # Create watchdog on start, and allow to shutdown the watchdog by setting it to None
         self._watchdog: Optional[Watchdog] = None
+        self._watchdog_task: Optional[asyncio.Task] = None
 
         self._plugin_manager: WechatyPluginManager = WechatyPluginManager(self)
 
+    @property
+    def puppet(self) -> Puppet:
+        """
+        Always expected to return a non-null puppet instance, or raise an error.
+        :return:
+        """
+        if not self._puppet:
+            raise WechatyStatusError('Wechaty puppet not loaded!')
+        return self._puppet
+
     @staticmethod
     def _load_puppet(options: WechatyOptions) -> Puppet:
         """
@@ -163,7 +190,7 @@ class Wechaty(AsyncIOEventEmitter):
         :return:
         """
         if options.puppet is None:
-            raise Exception('puppet not exist')
+            raise WechatyConfigurationError('puppet not exist')
 
         if isinstance(options.puppet, Puppet):
             return options.puppet
@@ -178,18 +205,18 @@ class Wechaty(AsyncIOEventEmitter):
             #
             hostie_module = __import__('wechaty_puppet_hostie')
             if not hasattr(hostie_module, 'HostiePuppet'):
-                raise Exception('HostiePuppet not exist in '
-                                'wechaty-puppet-hostie')
+                raise WechatyConfigurationError('HostiePuppet not exist in '
+                                                'wechaty-puppet-hostie')
 
             hostie_puppet_class = getattr(hostie_module, 'HostiePuppet')
             if not issubclass(hostie_puppet_class, Puppet):
-                raise TypeError(f'Type {hostie_puppet_class} '
-                                f'is not correct')
+                raise WechatyConfigurationError(f'Type {hostie_puppet_class} '
+                                                f'is not correct')
 
             return hostie_puppet_class(options.puppet_options)
 
-        raise TypeError('puppet expected type is [Puppet, '
-                        'PuppetModuleName(str)]')
+        raise WechatyConfigurationError('puppet expected type is [Puppet, '
+                                        'PuppetModuleName(str)]')
 
     def __str__(self):
         """str format of the Room object"""
@@ -348,34 +375,65 @@ class Wechaty(AsyncIOEventEmitter):
         start wechaty bot
         :return:
         """
-        await self.init_puppet()
-        await self.init_puppet_event_bridge(self.puppet)
 
         async def start_watchdog():
-            self._watchdog = Watchdog(DEFAULT_TIMEOUT)
-            watch_dog_id = id(self._watchdog)
-            food = WatchdogFood(timeout=3)
+            try:
+                self._watchdog = Watchdog(DEFAULT_TIMEOUT)
+                watch_dog_id = id(self._watchdog)
+                food = WatchdogFood(timeout=3)
 
-            async def ask_for_food(last_food, last_feed):
-                log.debug('dog ask for food <%s> <%s> ...',
-                          last_food, last_feed)
-                await self.puppet.ding()
+                async def ask_for_food(last_food, last_feed):
+                    log.debug('dog ask for food <%s> <%s> ...',
+                              last_food, last_feed)
+                    # Cancel when stopped, or the puppet.ding() call will got an error
+                    if self.puppet and self._watchdog and id(self._watchdog) == watch_dog_id:
+                        await self.puppet.ding()
 
-            self._watchdog.on('sleep', ask_for_food)
-            self._watchdog.feed(food)
-            # If we set _watchdog to None or a new one, this loop should break.
-            while self._watchdog and id(self._watchdog) == watch_dog_id:
-                log.debug('bot tick <%s>', datetime.now())
-                await self._watchdog.sleep()
-                is_death = self._watchdog.starved_to_death()
-                if is_death:
-                    await self.restart()
-                    break
+                self._watchdog.on('sleep', ask_for_food)
+                self._watchdog.feed(food)
+                # If we set _watchdog to None or a new one, this loop should break.
+                while self._watchdog and id(self._watchdog) == watch_dog_id:
+                    log.debug('bot tick <%s>', datetime.now())
+                    await self._watchdog.sleep()
+                    is_death = self._watchdog.starved_to_death()
+                    if is_death:
+                        await self.restart()
+                        break
+            except asyncio.CancelledError:
+                # When we stop the bot, we call cancel on the self._watchdog_task.
+                # In this case, a CancelledError will raise then, just exit this function
+                # to terminate the watchdog
+                pass
 
-        loop = asyncio.get_event_loop()
-        asyncio.run_coroutine_threadsafe(start_watchdog(), loop)
-        log.info('starting ...')
-        await self.puppet.start()
+        # If the network is shut-down, we should catch the connection
+        # error and restart after a minute.
+        try:
+
+            await self.init_puppet()
+            await self.init_puppet_event_bridge(self.puppet)
+
+            log.info('starting puppet ...')
+            await self.puppet.start()
+
+            self.started = True
+
+            self._watchdog_task = asyncio.create_task(start_watchdog())
+
+        except (requests.exceptions.ConnectionError, StreamTerminatedError, OSError):
+
+            log.error('The network is not good, the bot will try to restart after 60 seconds.')
+            await asyncio.sleep(60)
+            await self.restart()
+
+        except WechatyPuppetError:
+            traceback.print_exc()
+            loop = asyncio.get_event_loop()
+            loop.stop()
+
+        # except Exception:
+        #     traceback.print_exc()
+        #     loop = asyncio.get_event_loop()
+        #     loop.stop()
 
     async def restart(self):
         """restart the wechaty bot"""
@@ -582,7 +640,7 @@ class Wechaty(AsyncIOEventEmitter):
             elif event_name == 'reset':
                 log.info('receive <reset> event <%s>')
             else:
-                raise ValueError(f'event_name <{event_name}> unsupported!')
+                raise WechatyOperationError(f'event_name <{event_name}> unsupported!')
 
             log.info('initPuppetEventBridge() puppet.on(%s) (listenerCount:%s) '
                      'registering...',
@@ -596,8 +654,11 @@ class Wechaty(AsyncIOEventEmitter):
         """
         init puppet grpc connection
         """
+        # Recreate puppet instance
+        self._puppet = self._load_puppet(self._options)
+
         # Using metaclass to create a dynamic subclass to server multi bot instances.
-        meta_info = dict(_puppet=self.puppet, _wechaty=self)
+        meta_info = dict(_puppet=self.puppet, _wechaty=self, abstract=False)
         self.Contact = type('Contact', (Contact,), meta_info)
         self.ContactSelf = type('ContactSelf', (ContactSelf,), meta_info)
         self.Favorite = type('Favorite', (Favorite,), meta_info)
@@ -614,10 +675,34 @@ class Wechaty(AsyncIOEventEmitter):
         stop the wechaty
         """
         log.info('wechaty is stoping ...')
+        self.started = False
+
         # Shutdown the watchdog
-        self._watchdog = None
+        if self._watchdog:
+            log.info('stopping - unsubscribe watchdog events')
+            self._watchdog.remove_all_listeners()
+            log.info('stopping - unset watchdog')
+            self._watchdog = None
+        if self._watchdog_task:
+            log.info('stopping - stop watchdog task')
+            self._watchdog_task.cancel()
+            log.info('stopping - unset watchdog task')
+            self._watchdog_task = None
+
         # deinit puppet
-        await self.puppet.stop()
+        log.info('stopping - stop puppet')
+        try:
+            await self.puppet.stop()
+        except OSError as e:
+            # OSError: [Errno 101] Network is unreachable
+            if e.errno == 101:
+                pass
+            else:
+                raise e
+
+        log.info('stopping - unset puppet')
+        self._puppet = None
+        log.info('wechaty has been stopped gracefully!')
 
     def user_self(self) -> ContactSelf:
         """
